local ChatAddon

local KatiaChatter = {}

function KatiaChatter:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function KatiaChatter:Init()
	Apollo.RegisterAddon(self, false, "", {})
end

function KatiaChatter:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("KatiaChatter.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	self.nearbyPlayers = {}
	self.open = false
	self.nicknames = {}
	for name in string.gmatch(GameLib.GetPlayerCharacterName(), "%S+") do
		table.insert(self.nicknames, string.lower(name))
	end

	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
	Apollo.RegisterEventHandler("ChatMessage", "OnChat", self)
	Apollo.RegisterSlashCommand("kchat", "OnOpen", self)
	Apollo.RegisterSlashCommand("kchatter", "OnOpen", self)
	Apollo.RegisterSlashCommand("katiachat", "OnOpen", self)
	Apollo.RegisterSlashCommand("katiachatter", "OnOpen", self)
	
	self.chatConnectTimer = ApolloTimer.Create(1, false, "GetChatAddon", self)
end

function KatiaChatter:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndListen = Apollo.LoadForm(self.xmlDoc, "Listeners", nil, self)
		if self.wndListen == nil then
			Apollo.AddAddonErrorText(self, "Could not load the window for some reason.")
			return
		end
		
	    self.wndListen:Show(false, true)
	end
end

function KatiaChatter:OnOpen()
	self.wndListen:Invoke()
	self.open = true
	for _, player in pairs(self.nearbyPlayers) do
		player.wnd:Show(not player.wnd:GetUnit():IsOccluded(), true)
	end
end

function KatiaChatter:OnClose()
	self.wndListen:Close()
	self.open = false
	for _, player in pairs(self.nearbyPlayers) do
		player.wnd:Show(false, true)
	end
end

function KatiaChatter:OnUnitCreated(unit)
	if unit == nil or unit:GetName() == "" or not unit:IsACharacter() or unit:GetName() == GameLib.GetPlayerCharacterName() then
		return
	end

	if self.nearbyPlayers[unit:GetName()] == nil then
		local newPlayer = {}
		newPlayer.listening = false
		newPlayer.name = unit:GetName()
		newPlayer.unit = unit
		newPlayer.wnd = Apollo.LoadForm(self.xmlDoc, "PlayerWindow", nil, self)
		newPlayer.wnd:SetUnit(unit)
		newPlayer.wnd:SetData(newPlayer)
		newPlayer.wnd:Show(not unit:IsOccluded() and self.open, true)
		newPlayer.DropOne = function(player)
			table.remove(player.queued, 1)
			if #player.queued <= 0 then
				player.queued = nil
				player.wnd:FindChild("Attention"):Show(false, true)
			end
		end
		self.nearbyPlayers[unit:GetName()] = newPlayer
	end
end

function KatiaChatter:OnUnitDestroyed(unit)
	if unit == nil or unit:GetName() == "" then
		return
	end

	local player = self.nearbyPlayers[unit:GetName()]
	if player then
		player.wnd:Destroy()
		if player.queued then
			for _, queue in ipairs(player.queued) do
				queue.timer:Stop()
			end
		end
		if player.listWnd then
			player.listWnd:Destroy()
		end
		self.wndListen:FindChild("List"):ArrangeChildrenVert()
		self.nearbyPlayers[unit:GetName()] = nil
	end
end

function KatiaChatter:GetChatAddon()
	if ChatAddon == nil then
		ChatAddon = Apollo.GetAddon("ChatLog")
		self.chatConnectTimer = ApolloTimer.Create(1, false, "GetChatAddon", self)
	end
end

function KatiaChatter:AddMsg(chan, msg)
	if not ChatAddon then return end
	for key, wndChat in pairs(ChatAddon.tChatWindows) do
		if wndChat:GetText() == "KC" and not wndChat:GetData().tViewedChannels[chan:GetUniqueId()] then
			local tQueuedMessage = {}
			tQueuedMessage.tMessage = msg
			tQueuedMessage.eChannelType = chan:GetType()
			tQueuedMessage.strChannelName = chan:GetName()
			tQueuedMessage.strChannelCommand = chan:GetCommand()
			tQueuedMessage.idChannel = chan:GetUniqueId()
			wndChat:GetData().tMessageQueue:Push(tQueuedMessage)
		end
	end
	ChatAddon.bQueuedMessages = true
end

function KatiaChatter:PlayerClicked(wndHandler, wndControl)
	local player = wndHandler:GetParent():GetData()
	if not player then return end
	if player.listening then
		player.listening = false
		player.wnd:FindChild("Listening"):SetBGColor("red")
		if player.listWnd then
			player.listWnd:Destroy()
		end
	else
		player.listening = true
		player.wnd:FindChild("Listening"):SetBGColor("green")
		if player.queued then
			for _, queue in ipairs(player.queued) do
				self:AddMsg(queue.chan, queue.msg)
				queue.timer:Stop()
			end
			player.queued = nil
			player.wnd:FindChild("Attention"):Show(false, true)
		end
		player.listWnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndListen:FindChild("List"), self)
		player.listWnd:SetText(player.name)
		player.listWnd:SetData(player)
	end
	self.wndListen:FindChild("List"):ArrangeChildrenVert()
end

function KatiaChatter:OnListSelect(wndHandler, wndControl)
	local player = wndHandler:GetData()
	player.listening = false
	player.wnd:FindChild("Listening"):SetBGColor("red")
	player.listWnd:Destroy()
	self.wndListen:FindChild("List"):ArrangeChildrenVert()
end

function KatiaChatter:OcclusionChange(wndHandler, wndControl, bOccluded)
	wndHandler:Show(not bOccluded and self.open, true)
end

function KatiaChatter:HasMe(text)
	for _, nickname in ipairs(self.nicknames) do
		if string.find(text, nickname) then return true end
	end
	return false
end

function KatiaChatter:OnChat(chan, msg)
	if chan == nil or msg == nil then
		return
	end
	
	local player = self.nearbyPlayers[msg.strSender]
	local text = msg.arMessageSegments[1].strText
	if (player and player.listening) or msg.strSender == GameLib.GetPlayerCharacterName() then
		self:AddMsg(chan, msg)
	elseif (player and player.unit:GetTarget() and player.unit:GetTarget():IsThePlayer()) or self:HasMe(string.lower(text)) and (chan:GetName() == "Say" or chan:GetName() == "Emote") then
		msg.arMessageSegments[1].strText = text .. " (*)"
		self:AddMsg(chan, msg)
	elseif player then
		if not player.queued then
			player.queued = {}
			player.wnd:FindChild("Attention"):Show(true, true)
		end
		table.insert(player.queued, {
			chan = chan,
			msg = msg,
			timer = ApolloTimer.Create(20, false, "DropOne", player)
		})
	end
end

local KatiaChatterInst = KatiaChatter:new()
KatiaChatterInst:Init()